# This file is automatically generated.
# DO NOT MODIFY THIS FILE MANUALLY.

ENDPOINTS = {
    ("POST", "/slurm/v0.0.41/job/submit"): (
        "v0_0_41",
        {
            "path": "PostPathSlurmJobSubmit",
            "query": "PostQuerySlurmJobSubmit",
            "body": "PostBodySlurmJobSubmit",
            "200": "PostResponse200SlurmJobSubmit",
            "default": "PostResponseDefaultSlurmJobSubmit",
        },
    ),
    ("GET", "/slurm/v0.0.41/job/{job_id}"): (
        "v0_0_41",
        {
            "path": "GetPathSlurmJobJobid",
            "query": "GetQuerySlurmJobJobid",
            "body": "GetBodySlurmJobJobid",
            "200": "GetResponse200SlurmJobJobid",
            "default": "GetResponseDefaultSlurmJobJobid",
        },
    ),
    ("DELETE", "/slurm/v0.0.41/job/{job_id}"): (
        "v0_0_41",
        {
            "path": "DeletePathSlurmJobJobid",
            "query": "DeleteQuerySlurmJobJobid",
            "body": "DeleteBodySlurmJobJobid",
            "200": "DeleteResponse200SlurmJobJobid",
            "default": "DeleteResponseDefaultSlurmJobJobid",
        },
    ),
    ("GET", "/slurm/v0.0.41/jobs"): (
        "v0_0_41",
        {
            "path": "GetPathSlurmJobs",
            "query": "GetQuerySlurmJobs",
            "body": "GetBodySlurmJobs",
            "200": "GetResponse200SlurmJobs",
            "default": "GetResponseDefaultSlurmJobs",
        },
    ),
    ("POST", "/slurm/v0.0.40/job/submit"): (
        "v0_0_40",
        {
            "path": "PostPathSlurmJobSubmit",
            "query": "PostQuerySlurmJobSubmit",
            "body": "PostBodySlurmJobSubmit",
            "200": "PostResponse200SlurmJobSubmit",
            "default": "PostResponseDefaultSlurmJobSubmit",
        },
    ),
    ("GET", "/slurm/v0.0.40/job/{job_id}"): (
        "v0_0_40",
        {
            "path": "GetPathSlurmJobJobid",
            "query": "GetQuerySlurmJobJobid",
            "body": "GetBodySlurmJobJobid",
            "200": "GetResponse200SlurmJobJobid",
            "default": "GetResponseDefaultSlurmJobJobid",
        },
    ),
    ("DELETE", "/slurm/v0.0.40/job/{job_id}"): (
        "v0_0_40",
        {
            "path": "DeletePathSlurmJobJobid",
            "query": "DeleteQuerySlurmJobJobid",
            "body": "DeleteBodySlurmJobJobid",
            "200": "DeleteResponse200SlurmJobJobid",
            "default": "DeleteResponseDefaultSlurmJobJobid",
        },
    ),
    ("GET", "/slurm/v0.0.40/jobs"): (
        "v0_0_40",
        {
            "path": "GetPathSlurmJobs",
            "query": "GetQuerySlurmJobs",
            "body": "GetBodySlurmJobs",
            "200": "GetResponse200SlurmJobs",
            "default": "GetResponseDefaultSlurmJobs",
        },
    ),
    ("POST", "/slurm/v0.0.39/job/submit"): (
        "v0_0_39",
        {
            "path": "PostPathSlurmJobSubmit",
            "query": "PostQuerySlurmJobSubmit",
            "body": "PostBodySlurmJobSubmit",
            "200": "PostResponse200SlurmJobSubmit",
            "default": "PostResponseDefaultSlurmJobSubmit",
        },
    ),
    ("GET", "/slurm/v0.0.39/job/{job_id}"): (
        "v0_0_39",
        {
            "path": "GetPathSlurmJobJobid",
            "query": "GetQuerySlurmJobJobid",
            "body": "GetBodySlurmJobJobid",
            "200": "GetResponse200SlurmJobJobid",
            "default": "GetResponseDefaultSlurmJobJobid",
        },
    ),
    ("DELETE", "/slurm/v0.0.39/job/{job_id}"): (
        "v0_0_39",
        {
            "path": "DeletePathSlurmJobJobid",
            "query": "DeleteQuerySlurmJobJobid",
            "body": "DeleteBodySlurmJobJobid",
            "200": "DeleteResponse200SlurmJobJobid",
            "default": "DeleteResponseDefaultSlurmJobJobid",
        },
    ),
    ("GET", "/slurm/v0.0.39/jobs"): (
        "v0_0_39",
        {
            "path": "GetPathSlurmJobs",
            "query": "GetQuerySlurmJobs",
            "body": "GetBodySlurmJobs",
            "200": "GetResponse200SlurmJobs",
            "default": "GetResponseDefaultSlurmJobs",
        },
    ),
}
